require_relative 'stft_slice.rb'
require_relative '../audio/audio_file.rb'

module STFTSpectrogram
  # Represents time-frequency spectrogram
  class Spectrogram
    attr_reader :windows

    def initialize(audio, window_size, window_overlap)
      if window_size <= window_overlap
        raise ArgumentError, 'Window size cannot be <= window overlap!'
      end

      @windows = []
      @transformed = false
      audio.window_size = window_size
      audio.window_overlap = window_overlap
      split_to_windows(audio)
      audio.reset
    end

    def split_to_windows(audio)
      until audio.end?
        @windows.push(STFTSlice.new(audio.next_window, audio.current_time))
      end
    end

    # Performs FFT on all timed data windows
    def transform!
      @windows.each(&:do_fft!)
      @transformed = true
    end

    # Gets the highest frequency
    def max_freq
      return 0 unless transformed?
      @windows[0].max_freq
    end

    # Returns an array with all frequencies
    def freqs
      return [] unless transformed?
      @windows[0].freqs
    end

    # Sets low and high frequency filters
    def filter(low, high)
      @windows.each do |w|
        w.low = low
        w.high = high
      end
    end

    # Returns true if FFT was already performed
    def transformed?
      @transformed
    end

    private :split_to_windows
  end
end
